#include "../../../../../sql/dynamic/6.0.0.03.x4-server.sql"

#define COPY(newTable, oldTable, ...) \
		INSERT INTO X4_SCHEMA.newTable (##__VA_ARGS__) SELECT __VA_ARGS__ FROM X4PERM.oldTable

#ifdef MSSQL
	#define COPY_WITH_ID(newTable, oldTable, ...) \
		SET IDENTITY_INSERT X4_SCHEMA.newTable ON; __CR__\
		COPY(newTable, oldTable, __VA_ARGS__); __CR__\
		SET IDENTITY_INSERT X4_SCHEMA.newTable OFF
#else
	#define COPY_WITH_ID(newTable, oldTable, ...) COPY(newTable, oldTable, __VA_ARGS__)
#endif

#ifdef POSTGRESQL
	#define CONVERT_TO_BOOLEAN(column) \
		CASE WHEN column = 1 THEN true ELSE false END
#endif

#ifdef MSSQL
	ALTER ROLE db_datareader ADD MEMBER x4s;
GO
	ALTER ROLE db_datawriter ADD MEMBER x4s;
GO
#endif

COPY_WITH_ID(MODULES, X4P_MODULE, ID, NAME);
#ifdef POSTGRESQL
	INSERT INTO X4_SCHEMA.USERS
		(ID, FIRST_NAME, LAST_NAME, LOGIN_NAME, CREATED, CREATED_BY, LASTMODIFIED,
		LASTMODIFIED_BY, PASSWORD, PASSWORD_CHANGE_REQUIRED, PASSWORD_CHANGE_FORBIDDEN, ACTIVE,
		DELETED, EMAIL, METADATA)
	SELECT
		ID, FIRST_NAME, LAST_NAME, LOGIN_NAME, CREATED, CREATED_BY, LASTMODIFIED,
		LASTMODIFIED_BY, PASSWORD, CONVERT_TO_BOOLEAN(PASSWORD_CHANGE_REQUIRED), FALSE, CONVERT_TO_BOOLEAN(ACTIVE),
		DELETED, EMAIL, METADATA FROM X4PERM.X4P_USER;
#else
COPY_WITH_ID(USERS, X4P_USER, ID, FIRST_NAME, LAST_NAME, LOGIN_NAME, CREATED, CREATED_BY, LASTMODIFIED,
	LASTMODIFIED_BY, PASSWORD, PASSWORD_CHANGE_REQUIRED, ACTIVE, DELETED, EMAIL, METADATA);
#endif
COPY(EXTENDED_USER_STATES, X4P_EXTENDED_USER_STATE,
	USER_ID, FAILED_LOGIN_ATTEMPTS, LATEST_FAILED_LOGIN);
COPY_WITH_ID(PERMS, X4P_PERM, ID, OBJECT_TYPE, ACTION, DESCRIPTION, MODULE,
	PARAMETER, CREATED, CREATED_BY, NAME);
COPY_WITH_ID(PERM_TEMPLATES, X4P_PERM_TEMPLATE, ID, OBJECT_TYPE, ACTION,
	DESCRIPTION, MODULE, NAME);
COPY_WITH_ID(ROLES, X4P_ROLE, ID, NAME, DISPLAY_NAME, DESCRIPTION, DATA, MODULE,
	CREATED, CREATED_BY, LASTMODIFIED, LASTMODIFIED_BY);
COPY_WITH_ID(ROLE_PERM, X4P_ROLE_PERM, ID, ROLE_ID, PERMISSION_ID);
#ifdef POSTGRESQL
	INSERT INTO X4_SCHEMA.GROUPS
		(ID, NAME, DISPLAY_NAME, DESCRIPTION, CREATED,
		CREATED_BY, LASTMODIFIED, LASTMODIFIED_BY, PRIVATE, DATA)
	SELECT
		ID, NAME, DISPLAY_NAME, DESCRIPTION, CREATED,
		CREATED_BY, LASTMODIFIED, LASTMODIFIED_BY, CONVERT_TO_BOOLEAN(PRIVATE), DATA FROM X4PERM.X4P_GROUP;
#else
COPY_WITH_ID(GROUPS, X4P_GROUP, ID, NAME, DISPLAY_NAME, DESCRIPTION, CREATED,
	CREATED_BY, LASTMODIFIED, LASTMODIFIED_BY, PRIVATE, DATA);
#endif
COPY_WITH_ID(GROUP_ROLE, X4P_GROUP_ROLE, ID, GROUP_ID, ROLE_ID);
COPY_WITH_ID(USER_GROUP, X4P_USER_GROUP, ID, GROUP_ID, USER_ID);
COPY_WITH_ID(PASSWORD_POLICIES, X4P_PASSWORD_POLICIES, ID, VALID_FROM, CREATED_BY, MIN_LEN, MIN_DIGITS, MIN_UPPERCASE,
	MIN_LOWERCASE, MIN_SPECIAL_CHAR, FAIL_ATTEMPTS_TO_LOCK, LOCKING_PERIOD, MAX_PASSWORD_AGE, PASSWORD_HISTORY_ENTRIES);
COPY_WITH_ID(PASSWORD_HISTORY, X4P_PASSWORD_HISTORY, ID, USER_ID, PASSWORD, VALID_UNTIL);
COPY_WITH_ID(NAVIGATOR_CONFIGS, X4P_NAVIGATOR_CONFIG, ID, ID_OF_XML);
COPY_WITH_ID(NAVIGATOR_FOLDERS, X4P_NAVIGATOR_FOLDER, ID, CONFIG_ID, ID_OF_XML);
COPY_WITH_ID(LIST_PREFERENCES, X4P_LIST_PREFERENCE, ID, USER_ID, FOLDER_ID, DATA_NAME, DATA, RANK);

DROP TABLE X4PERM.X4P_PERM_TEMPLATE;
DROP TABLE X4PERM.X4P_ROLE_PERM;
DROP TABLE X4PERM.X4P_USER_GROUP;
DROP TABLE X4PERM.X4P_GROUP_ROLE;
DROP TABLE X4PERM.X4P_LIST_PREFERENCE;
DROP TABLE X4PERM.X4P_PERM;
DROP TABLE X4PERM.X4P_ROLE;
DROP TABLE X4PERM.X4P_GROUP;
DROP TABLE X4PERM.X4P_MODULE;
DROP TABLE X4PERM.X4P_PASSWORD_HISTORY;
DROP TABLE X4PERM.X4P_PASSWORD_POLICIES;
DROP TABLE X4PERM.X4P_EXTENDED_USER_STATE;
DROP TABLE X4PERM.X4P_USER;
DROP TABLE X4PERM.X4P_NAVIGATOR_FOLDER;
DROP TABLE X4PERM.X4P_NAVIGATOR_CONFIG;

#ifndef ORACLE
    DROP SCHEMA X4PERM;
#else
    DROP USER X4PERM CASCADE;
#endif