#include "include/schema.include"
#include "include/sequence.include"
#include "include/constraint.include"
#include "include/type.include"

CREATE TABLE X4_SCHEMA.MODULES
(
	ID X4_TYPE_ID,
	NAME X4_TYPE_VARCHAR(100),
	PK(MODULES_PK_ID, ID),
	UQ(MODULES_UNQ_NAME, NAME)
);
SEQ(ID,MODULES,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.USERS (
	ID X4_TYPE_ID,
	FIRST_NAME X4_TYPE_VARCHAR_NULLABLE(50),
	LAST_NAME X4_TYPE_VARCHAR(100),
	LOGIN_NAME X4_TYPE_VARCHAR(255),
	CREATED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	CREATED_BY X4_TYPE_FK_NULLABLE,
	LASTMODIFIED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	LASTMODIFIED_BY X4_TYPE_FK_NULLABLE,
	PASSWORD X4_TYPE_VARCHAR(100),
	PASSWORD_CHANGE_REQUIRED X4_TYPE_BOOLEAN,
	PASSWORD_CHANGE_FORBIDDEN X4_TYPE_BOOLEAN_WITH_DEFAULT(FALSE),
	ACTIVE X4_TYPE_BOOLEAN,
	VALID_FROM X4_TYPE_DATETIME_NULLABLE,
	VALID_UNTIL X4_TYPE_DATETIME_NULLABLE,
	DELETED X4_TYPE_LONG_NULLABLE DEFAULT 0,
	EMAIL X4_TYPE_VARCHAR_NULLABLE(255),
	METADATA X4_TYPE_VARCHAR_NULLABLE(700),
	CUSTOM_METADATA X4_TYPE_TEXT_NULLABLE,
	CERT_ALIAS X4_TYPE_VARCHAR_NULLABLE(255),
	CERT_FINGERPRINT X4_TYPE_VARCHAR_NULLABLE(255),
	PK(USERS_ID, ID),
	UQ(USERS_LOGIN_NAME, LOGIN_NAME, DELETED),
	FK(USERS_CREATED_BY,CREATED_BY,USERS,ID,X4_SCHEMA),
	FK(USERS_LASTMODIFIED_BY,LASTMODIFIED_BY,USERS,ID,X4_SCHEMA)
	);
SEQ(ID,USERS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.EXTENDED_USER_STATES (
	USER_ID X4_TYPE_FK,
	FAILED_LOGIN_ATTEMPTS X4_TYPE_INTEGER_WITH_DEFAULT(0),
	LATEST_FAILED_LOGIN X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	PK(EXTENDED_USER_STATES,USER_ID),
	FK(EXT_USER_STATES_USER_ID,USER_ID,USERS,ID,X4_SCHEMA) ON DELETE CASCADE
);

CREATE TABLE X4_SCHEMA.PERMS (
	ID X4_TYPE_ID,
	OBJECT_TYPE X4_TYPE_VARCHAR(100),
	ACTION X4_TYPE_VARCHAR(100),
	DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(500),
	MODULE X4_TYPE_FK,
	PARAMETER X4_TYPE_VARCHAR_NULLABLE(255),
	CREATED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	CREATED_BY X4_TYPE_FK_NULLABLE,
	NAME X4_TYPE_VARCHAR(100),
	PK(PERMS_ID,ID),
	UQ(PERMS_NAME,NAME),
	UQ(PERMS_PERMISSION, MODULE, OBJECT_TYPE, ACTION, PARAMETER),
	FK(PERMS_MODULE,MODULE,MODULES,ID,X4_SCHEMA),
	FK(PERMS_CREATED_BY,CREATED_BY,USERS,ID,X4_SCHEMA)
);
SEQ(ID,PERMS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.PERM_TEMPLATES
(
  	ID X4_TYPE_ID,
	OBJECT_TYPE X4_TYPE_VARCHAR(100),
	ACTION X4_TYPE_VARCHAR(100),
	DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(500),
	MODULE X4_TYPE_FK,
	NAME X4_TYPE_VARCHAR(100),
	PK(PERM_TEMPLATES,ID),
	UQ(PERM_TEMPLATES_PERMISSION,OBJECT_TYPE,ACTION,MODULE),
	FK(PERM_TEMPLATES_MODULE,MODULE,MODULES,ID,X4_SCHEMA)
);
SEQ(ID,PERM_TEMPLATES,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.ROLES(
	ID X4_TYPE_ID,
	NAME X4_TYPE_VARCHAR(200),
	DISPLAY_NAME X4_TYPE_VARCHAR(255),
	DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(500),
	DATA X4_TYPE_VARCHAR_NULLABLE(700),
	MODULE X4_TYPE_FK,
	CREATED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	CREATED_BY X4_TYPE_FK_NULLABLE,
	LASTMODIFIED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	LASTMODIFIED_BY X4_TYPE_FK_NULLABLE,
	PK(ROLES_ID,ID),
	UQ(ROLES_DISPLAY_NAME,DISPLAY_NAME),
	UQ(ROLES_NAME_DATA,NAME,DATA),
	FK(ROLES_MODULE,MODULE,MODULES,ID,X4_SCHEMA),
	FK(ROLES_CREATED_BY,CREATED_BY,USERS,ID,X4_SCHEMA),
	FK(ROLES_LASTMODOFIED_BY,LASTMODIFIED_BY,USERS,ID,X4_SCHEMA)
);
SEQ(ID,ROLES,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.ROLE_PERM (
	ID X4_TYPE_ID,
	ROLE_ID X4_TYPE_FK,
	PERMISSION_ID X4_TYPE_FK,
	PK(ROLE_PERM,ID),
	UQ(ROLE_PERM,ROLE_ID,PERMISSION_ID),
	FK(ROLE_PERM_ROLE_ID,ROLE_ID,ROLES,ID,X4_SCHEMA),
	FK(ROLE_PERM_PERM_ID,PERMISSION_ID,PERMS,ID,X4_SCHEMA)
);
SEQ(ID,ROLE_PERM,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.GROUPS(
	ID X4_TYPE_ID,
	NAME X4_TYPE_VARCHAR(200),
	DISPLAY_NAME X4_TYPE_VARCHAR(500),
	DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(500),
	CREATED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	CREATED_BY X4_TYPE_FK_NULLABLE,
	LASTMODIFIED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
	LASTMODIFIED_BY X4_TYPE_FK_NULLABLE,
	PRIVATE X4_TYPE_BOOLEAN_WITH_DEFAULT(FALSE),
	DATA X4_TYPE_VARCHAR_NULLABLE(700),
	CUSTOM_METADATA X4_TYPE_TEXT_NULLABLE,
	PK(GROUPS_ID,ID),
	UQ(GROUPS_DISPLAY_NAME,DISPLAY_NAME),
	UQ(GROUPS_NAME_DATA,NAME,DATA),
	FK(GROUPS_CREATED_BY,CREATED_BY,USERS,ID,X4_SCHEMA),
	FK(GROUPS_LASTMODIFIED_BY,LASTMODIFIED_BY,USERS,ID,X4_SCHEMA)
);
SEQ(ID,GROUPS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.GROUP_ROLE (
	ID X4_TYPE_ID,
	GROUP_ID X4_TYPE_FK,
	ROLE_ID X4_TYPE_FK,
	PK(GROUP_ROLE,ID),
	UQ(GROUP_ROLE,ROLE_ID,GROUP_ID),
	FK(GROUP_ROLE_ROLE_ID,ROLE_ID,ROLES,ID,X4_SCHEMA),
	FK(GROUP_ROLE_GROUP_ID,GROUP_ID,GROUPS,ID,X4_SCHEMA)
);
SEQ(ID,GROUP_ROLE,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.USER_GROUP (
	ID X4_TYPE_ID,
	GROUP_ID X4_TYPE_FK,
	USER_ID X4_TYPE_FK,
	PK(USER_GROUP,ID),
	UQ(USER_GROUP,USER_ID,GROUP_ID),
	FK(USER_GROUP_USER_ID,USER_ID,USERS,ID,X4_SCHEMA),
	FK(USER_GROUP_GROUP_ID,GROUP_ID,GROUPS,ID,X4_SCHEMA)
);
SEQ(ID,USER_GROUP,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.PASSWORD_POLICIES (
	ID X4_TYPE_ID,
	VALID_FROM X4_TYPE_DATETIME_WITH_DEFAULT(NOW),
	CREATED_BY X4_TYPE_FK,
	MIN_LEN X4_TYPE_INTEGER,
	MIN_DIGITS X4_TYPE_INTEGER,
	MIN_UPPERCASE X4_TYPE_INTEGER,
	MIN_LOWERCASE X4_TYPE_INTEGER,
	MIN_SPECIAL_CHAR X4_TYPE_INTEGER,
	FAIL_ATTEMPTS_TO_LOCK X4_TYPE_INTEGER,
	LOCKING_PERIOD X4_TYPE_LONG_NULLABLE,
	MAX_PASSWORD_AGE X4_TYPE_LONG_NULLABLE,
	PASSWORD_HISTORY_ENTRIES X4_TYPE_INTEGER,
	PK(PASSWORD_POLICIES,ID),
	FK(PASSWORD_POLICIES_USER,CREATED_BY,USERS,ID,X4_SCHEMA),
	UQ(PASSWORD_POLICIES_VALID, VALID_FROM)
);
SEQ(ID,PASSWORD_POLICIES,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.PASSWORD_HISTORY (
	ID X4_TYPE_ID,
	USER_ID X4_TYPE_FK,
	PASSWORD X4_TYPE_VARCHAR(100),
	VALID_UNTIL X4_TYPE_DATETIME_WITH_DEFAULT(NOW),
	PK(PASSWORD_HISTORY,ID),
	FK(PASSWORD_HISTORY_USER_ID,USER_ID,USERS,ID,X4_SCHEMA)
);
SEQ(ID,PASSWORD_HISTORY,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.NAVIGATOR_CONFIGS
(
	ID X4_TYPE_ID,
	ID_OF_XML X4_TYPE_VARCHAR(100),
	PK(NAVIGATOR_CONFIGS,ID),
	UQ(NAVIGATOR_CONFIGS,ID_OF_XML)
);
SEQ(ID,NAVIGATOR_CONFIGS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.NAVIGATOR_FOLDERS
(
	ID X4_TYPE_ID,
	CONFIG_ID X4_TYPE_FK,
	ID_OF_XML X4_TYPE_VARCHAR(100),
	PK(NAVIGATOR_FOLDERS,ID),
	FK(NAVIGATOR_FOLDERS,CONFIG_ID,NAVIGATOR_CONFIGS,ID,X4_SCHEMA) ON DELETE CASCADE,
	UQ(NAVIGATOR_FOLDERS,CONFIG_ID,ID_OF_XML)
);
SEQ(ID,NAVIGATOR_FOLDERS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.LIST_PREFERENCES
(
	ID X4_TYPE_ID,
	USER_ID X4_TYPE_FK,
	FOLDER_ID X4_TYPE_FK,
	DATA_NAME X4_TYPE_VARCHAR(800),
	DATA X4_TYPE_TEXT,
	RANK X4_TYPE_INTEGER,
	PK(LIST_PREFERENCES, ID),
	FK(LIST_PREFERENCES_USER_ID,USER_ID,USERS,ID,X4_SCHEMA) ON DELETE CASCADE,
	FK(LIST_PREFERENCES_FOLDER_ID,FOLDER_ID,NAVIGATOR_FOLDERS,ID,X4_SCHEMA) ON DELETE CASCADE,
	UQ(LIST_PREFERENCES_NAME,USER_ID,FOLDER_ID,DATA_NAME)
);
SEQ(ID,LIST_PREFERENCES,X4_SCHEMA);
