#include "include/schema.include"
#include "include/constraint.include"
#include "include/type.include"
#include "include/sequence.include"

#ifdef MSSQL
	#define DROP_INDEX(name,table) DROP INDEX name ON X4_SCHEMA.table
#elif ORACLE
	#define DROP_INDEX(name,table) -- noop
#else
	#define DROP_INDEX(name,table) DROP INDEX X4_SCHEMA.name
#endif

#ifndef ORACLE
	#define DROP_COLUMN_OID(table) \
		DROP_INDEX(IDX_##table##_OID,table); __CR__ \
		ALTER TABLE X4_SCHEMA.table DROP COLUMN OID
#else 
	#define DROP_COLUMN_OID(table) \
		ALTER TABLE X4_SCHEMA.table DROP COLUMN OID
#endif

ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT FK_X4_TASKS_PARENT_ID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN PARENT_ID;
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT FK_X4_CASES_PARENT_ID;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN PARENT_ID;

ALTER TABLE X4_SCHEMA.X4_CASE_DEFINITIONS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_CASES ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_DATA ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_FEATURES ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_JOURNALS ADD UUID X4_TYPE_UUID;

ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_TASKS ADD UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_TASKS ADD OWNER X4_TYPE_VARCHAR_NULLABLE(100);
ALTER TABLE X4_SCHEMA.X4_TASKS ADD ASSIGNED_TYPE X4_TYPE_VARCHAR_NULLABLE(5);
ALTER TABLE X4_SCHEMA.X4_TASKS ADD ASSIGNED_USER_GROUP X4_TYPE_VARCHAR_NULLABLE(100);
ALTER TABLE X4_SCHEMA.X4_TASKS ADD LOCKED X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_TASKS ADD LOCKED_BY X4_TYPE_VARCHAR_NULLABLE(50);
ALTER TABLE X4_SCHEMA.X4_TASKS ADD HUMAN_TASK_UUID X4_TYPE_UUID;
ALTER TABLE X4_SCHEMA.X4_TASKS ADD COMPLETED X4_TYPE_DATETIME_NULLABLE;

ALTER TABLE X4_SCHEMA.X4_COMMENTS ADD OBJECT_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_DATA ADD OBJECT_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES ADD OBJECT_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_FEATURES ADD OBJECT_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_JOURNALS ADD OBJECT_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_OBJECT_TO_PID ADD OBJECT_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_TASKS ADD CASE_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_CASES ADD CASE_DEF_UUID X4_TYPE_FK_UUID;

ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS ADD CASE_DEF_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS ADD CATEGORY_DEF_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS ADD GROUP_DEF_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS ADD CASE_DEF_UUID X4_TYPE_FK_UUID;
ALTER TABLE X4_SCHEMA.X4_FEATURES ADD FEATURE_DEF_UUID X4_TYPE_FK_UUID;

#ifdef MSSQL
GO
#endif

-- repair uuid references
INSERT INTO X4_SCHEMA.X4_CASE_DEFINITIONS (NAME,URL) VALUES ('UNDEFINED','UNDEFINED');
UPDATE X4_SCHEMA.X4_CASES SET CASE_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_CASE_DEFINITIONS WHERE NAME = 'UNDEFINED' AND URL = 'UNDEFINED') WHERE CASE_DEFINITION_ID IS NULL;
UPDATE X4_SCHEMA.X4_CASES SET UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_CASES.ID);
UPDATE X4_SCHEMA.X4_CASES SET CASE_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_CASE_DEFINITIONS WHERE X4_CASE_DEFINITIONS.ID = X4_CASES.CASE_DEFINITION_ID) WHERE CASE_DEFINITION_ID IS NOT NULL;
UPDATE X4_SCHEMA.X4_CATEGORY_DEFINITIONS SET CASE_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_CASE_DEFINITIONS WHERE X4_CASE_DEFINITIONS.ID = X4_CATEGORY_DEFINITIONS.CASE_DEFINITION_ID);
UPDATE X4_SCHEMA.X4_COMMENTS SET OBJECT_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_COMMENTS.OID);
UPDATE X4_SCHEMA.X4_DATA SET UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_DATA.ID);
UPDATE X4_SCHEMA.X4_DATA SET OBJECT_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_DATA.OID);
UPDATE X4_SCHEMA.X4_DATA_REFERENCES SET UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_DATA_REFERENCES.ID);
UPDATE X4_SCHEMA.X4_DATA_REFERENCES SET OBJECT_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_DATA_REFERENCES.OID);
UPDATE X4_SCHEMA.X4_FEATURE_DEFINITIONS SET GROUP_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_GROUP_DEFINITIONS WHERE X4_GROUP_DEFINITIONS.ID = X4_FEATURE_DEFINITIONS.GROUP_ID);
UPDATE X4_SCHEMA.X4_FEATURE_DEFINITIONS SET CASE_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_CASE_DEFINITIONS WHERE X4_CASE_DEFINITIONS.ID = X4_FEATURE_DEFINITIONS.CASE_DEFINITION_ID);
UPDATE X4_SCHEMA.X4_FEATURES SET FEATURE_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_FEATURE_DEFINITIONS WHERE X4_FEATURE_DEFINITIONS.ID = X4_FEATURES.FEATURE_DEF_ID);
UPDATE X4_SCHEMA.X4_FEATURES SET OBJECT_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_FEATURES.OID);
UPDATE X4_SCHEMA.X4_GROUP_DEFINITIONS SET CATEGORY_DEF_UUID = (SELECT UUID FROM X4_SCHEMA.X4_CATEGORY_DEFINITIONS WHERE X4_CATEGORY_DEFINITIONS.ID = X4_GROUP_DEFINITIONS.CATEGORY_ID);
UPDATE X4_SCHEMA.X4_JOURNALS SET OBJECT_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_JOURNALS.OID);
UPDATE X4_SCHEMA.X4_TASKS SET UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_TASKS.ID);
UPDATE X4_SCHEMA.X4_TASKS SET CASE_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_TASKS.CASE_ID);
UPDATE X4_SCHEMA.X4_OBJECT_TO_PID SET OBJECT_UUID = (SELECT UUID FROM X4_SCHEMA.X4_OBJECT_IDS WHERE X4_OBJECT_IDS.ID = X4_OBJECT_TO_PID.OID) WHERE OID IS NOT NULL;

-- drop old fk constraints
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT FK_X4_CASES_OID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP CONSTRAINT FK_X4_COMMENTS_OID;
ALTER TABLE X4_SCHEMA.X4_DATA DROP CONSTRAINT FK_X4_DATA_OID;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP CONSTRAINT FK_X4_DATA_REFERENCES_OID;
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP CONSTRAINT FK_X4_FEATURES_OID;
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP CONSTRAINT FK_X4_JOURNALS_OID;
ALTER TABLE X4_SCHEMA.X4_OBJECT_TO_PID DROP CONSTRAINT FK_X4_PID_OID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT FK_X4_TASKS_OID;

ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT FK_X4_TASKS_CASE_ID;

ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT FK_X4_CASE_DEF;
ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS DROP CONSTRAINT FK_X4_CATEGORY_DEF_CASE_DEF;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS DROP CONSTRAINT FK_X4_GROUP_CATEGORY_DEF_ID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP CONSTRAINT FK_X4_FEATURE_DEF_CASE_DEF;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP CONSTRAINT FK_X4_FEATURE_DEF_GROUP_ID;
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP CONSTRAINT FK_X4_FEATURE_DEF_ID;

ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS DROP CONSTRAINT UQ_X4_CATEGORY_DEFINITIONS;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS DROP CONSTRAINT UQ_X4_GROUP_DEFINITIONS;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP CONSTRAINT UQ_X4_FEATURE_DEFINITIONS;
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP CONSTRAINT UQ_X4_FEATURES;

-- drop old pk constraints
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT PK_X4_CASES_ID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP CONSTRAINT PK_X4_COMMENTS_ID;
ALTER TABLE X4_SCHEMA.X4_DATA DROP CONSTRAINT PK_X4_DATA;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP CONSTRAINT PK_X4_DATA_REFERENCES;
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP CONSTRAINT PK_X4_FEATURES;
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP CONSTRAINT PK_X4_JOURNALS_ID;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS DROP CONSTRAINT PK_X4_OBJECT_IDS;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT PK_X4_TASKS_ID;

ALTER TABLE X4_SCHEMA.X4_CASE_DEFINITIONS DROP CONSTRAINT PK_X4_CASE_DEF_ID;
ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS DROP CONSTRAINT PK_X4_CATEGORY_DEF_ID;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS DROP CONSTRAINT PK_X4_GROUP_DEF_ID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP CONSTRAINT PK_X4_FEATURE_DEF_ID;

DROP_SEQ(ID, X4_COMMENTS, X4_SCHEMA);
DROP_SEQ(ID, X4_DATA, X4_SCHEMA);
DROP_SEQ(ID, X4_DATA_REFERENCES, X4_SCHEMA);
DROP_SEQ(ID, X4_FEATURES, X4_SCHEMA);
DROP_SEQ(ID, X4_JOURNALS, X4_SCHEMA);
DROP_SEQ(ID, X4_OBJECT_IDS, X4_SCHEMA);
DROP_SEQ(ID, X4_CASE_DEFINITIONS, X4_SCHEMA);
DROP_SEQ(ID, X4_CATEGORY_DEFINITIONS, X4_SCHEMA);
DROP_SEQ(ID, X4_GROUP_DEFINITIONS, X4_SCHEMA);
DROP_SEQ(ID, X4_FEATURE_DEFINITIONS, X4_SCHEMA);

-- add new pk constraints
ALTER_TABLE_PK(X4_CASES, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_COMMENTS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_DATA, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_DATA_REFERENCES, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_FEATURES, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_JOURNALS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_TASKS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_CASE_DEFINITIONS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_CATEGORY_DEFINITIONS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_GROUP_DEFINITIONS, UUID, X4_SCHEMA);
ALTER_TABLE_PK(X4_FEATURE_DEFINITIONS, UUID, X4_SCHEMA);

-- add new fk constraints
ALTER_TABLE_FK(X4_CASES_UUID, X4_CASES, UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_COMMENTS_UUID, X4_COMMENTS, OBJECT_UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_DATA_UUID, X4_DATA, OBJECT_UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_DATA_REF_UUID, X4_DATA_REFERENCES, OBJECT_UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_FEATURES_UUID, X4_FEATURES, OBJECT_UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_JOURNALS_UUID, X4_JOURNALS, OBJECT_UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_OBJECT_REF, X4_OBJECT_TO_PID, OBJECT_UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_TASKS_UUID, X4_TASKS, UUID, X4_OBJECT_IDS, UUID, X4_SCHEMA);

ALTER_TABLE_FK(X4_TASKS_CASE_UUID, X4_TASKS, CASE_UUID, X4_CASES, UUID, X4_SCHEMA);

ALTER_TABLE_FK(X4_CASES_CASE_DEF_UUID, X4_CASES, CASE_DEF_UUID, X4_CASE_DEFINITIONS, UUID, X4_SCHEMA) ON DELETE CASCADE;
ALTER_TABLE_FK(X4_CAT_DEF_CASE_DEF_UUID, X4_CATEGORY_DEFINITIONS, CASE_DEF_UUID, X4_CASE_DEFINITIONS, UUID, X4_SCHEMA) ON DELETE CASCADE;
ALTER_TABLE_FK(X4_GRO_DEF_CAT_DEF_UUID, X4_GROUP_DEFINITIONS, CATEGORY_DEF_UUID, X4_CATEGORY_DEFINITIONS, UUID, X4_SCHEMA) ON DELETE CASCADE;
ALTER_TABLE_FK(X4_FEA_DEF_GRO_DEF_UUID, X4_FEATURE_DEFINITIONS, GROUP_DEF_UUID, X4_GROUP_DEFINITIONS, UUID, X4_SCHEMA) ON DELETE CASCADE;
ALTER_TABLE_FK(X4_FEA_DEF_CASE_DEF_UUID, X4_FEATURE_DEFINITIONS, CASE_DEF_UUID, X4_CASE_DEFINITIONS, UUID, X4_SCHEMA);
ALTER_TABLE_FK(X4_FEATURE_FEA_DEF_UUID, X4_FEATURES, FEATURE_DEF_UUID, X4_FEATURE_DEFINITIONS, UUID, X4_SCHEMA) ON DELETE CASCADE;

ALTER_TABLE_UQ(X4_CATEGORY_DEFINITIONS, X4_CATEGORY_DEFINITIONS, X4_SCHEMA, NAME, CASE_DEF_UUID);
ALTER_TABLE_UQ(X4_GROUP_DEFINITIONS, X4_GROUP_DEFINITIONS, X4_SCHEMA, NAME, CATEGORY_DEF_UUID);
ALTER_TABLE_UQ(X4_FEATURE_DEFINITIONS, X4_FEATURE_DEFINITIONS, X4_SCHEMA, NAME, CASE_DEF_UUID);
ALTER_TABLE_UQ(X4_FEATURES, X4_FEATURES, X4_SCHEMA, OBJECT_UUID, FEATURE_DEF_UUID);

-- drop old id columns
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT UQ_X4_CASES_DOMAIN_ID;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN DOMAIN_ID;
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT CHK_CASES_ASSIGNMENT;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN AGENT;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN AGENT_GROUP;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN PRIO;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN DEADLINE;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN REMINDER;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT UQ_X4_TASKS_DOMAIN_ID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN DOMAIN_ID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT CHK_TASKS_ASSIGNMENT;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN AGENT;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN AGENT_GROUP;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN PRIO;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS DROP COLUMN LOCKED;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS DROP COLUMN LOCKED_BY;

ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP COLUMN ID;
DROP_COLUMN_OID(X4_COMMENTS);
ALTER TABLE X4_SCHEMA.X4_DATA DROP COLUMN ID;
DROP_COLUMN_OID(X4_DATA);
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP COLUMN ID;
DROP_COLUMN_OID(X4_DATA_REFERENCES);
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP COLUMN ID;
DROP_COLUMN_OID(X4_JOURNALS);
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP COLUMN OID;
ALTER TABLE X4_SCHEMA.X4_FEATURES DROP COLUMN FEATURE_DEF_ID;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_OBJECT_TO_PID DROP COLUMN OID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN ID;
DROP_INDEX(IDX_X4_TASKS_CASE_ID,X4_TASKS);
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN CASE_ID;
ALTER TABLE X4_SCHEMA.X4_CASE_DEFINITIONS DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS DROP COLUMN CASE_DEFINITION_ID;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS DROP COLUMN CATEGORY_ID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP COLUMN GROUP_ID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP COLUMN CASE_DEFINITION_ID;
ALTER TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS DROP COLUMN SEARCHABLE;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN CASE_DEFINITION_ID;
#ifdef ORACLE
	#define TYPE "TYPE"
#endif
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN TYPE;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN TYPE;

-- Create new savepoint table for human tasks
CREATE TABLE X4_SCHEMA.X4_TASK_SAVEPOINTS
(
  UUID X4_TYPE_UUID,
  DATA X4_TYPE_BLOB,
  METADATA X4_TYPE_BLOB,
  PK(X4_TASK_SAVEPOINTS, UUID),
  FK(X4_TASK_SAVEPOINTS, UUID, X4_TASKS, UUID, X4_SCHEMA)
);