<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="3.0">
	<xsl:output media-type="text/xml" method="xml"></xsl:output>
	<xsl:variable name="process" select="." />
	<!-- Max ActionId which node has SVGId attribute to use for the calculation of the new Stop actions ID -->
	<xsl:variable name="maxActionId">
		<xsl:choose>
			<xsl:when test="/Workflow/Actions/Action[@SVGid]/@id">
				<xsl:value-of select="max(Workflow/Actions/Action[@SVGid]/@id)"></xsl:value-of>
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="stopToTransition">
		<xsl:apply-templates mode="stopToTransition" select="/Workflow/Graphics/svg/g/polyline[starts-with(@join-to, 'Element085')]"></xsl:apply-templates>
	</xsl:variable>
	<xsl:template match="/">
		<xsl:copy>
		<xsl:apply-templates select="@*"></xsl:apply-templates>
			<xsl:apply-templates select="node()"></xsl:apply-templates>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="Transition/To[@action='999']">
		<xsl:variable name="SVGid" select="../@SVGid"></xsl:variable>
		<xsl:copy>
			<xsl:apply-templates select="@*"></xsl:apply-templates>
			<xsl:if test="exists($stopToTransition/Transition[@SVGid=$SVGid])">
				<xsl:attribute name="action" select="$stopToTransition/Transition[@SVGid=$SVGid]/@actionId"></xsl:attribute>
			</xsl:if>
			<xsl:apply-templates select="node()"></xsl:apply-templates>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="/Workflow/Actions">
		<xsl:copy>
			<xsl:apply-templates select="@*"></xsl:apply-templates>
			<xsl:apply-templates select="node()"></xsl:apply-templates>
			<!-- Hide action for Start component only for WRF processes -->
			<xsl:if test="count(Action[Module='Start']) = 0">
				<Action id="0">
					<Name>Start</Name>
					<Comment></Comment>
					<Module>Start</Module>
					<Label>Start</Label>
					<Parameters></Parameters>
				</Action>
			</xsl:if>
			<xsl:for-each select="$stopToTransition/*">
				<Action id="{./@actionId}" SVGid="{./@stopElement}">
					<Name>Stop</Name>
					<Comment></Comment>
					<Module>Stop</Module>
					<Label>Stop</Label>
					<Parameters></Parameters>
				</Action>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="*">
		<xsl:copy>
			<xsl:apply-templates select="@*"></xsl:apply-templates>
			<xsl:apply-templates select="node()"></xsl:apply-templates>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="@*">
		<xsl:copy-of select="."></xsl:copy-of>
	</xsl:template>
	<xsl:template
		match="text()|processing-instruction()|comment()">
		<xsl:copy-of select="."></xsl:copy-of>
	</xsl:template>
	<xsl:template
		match="/Workflow/Graphics/svg/g/polyline[starts-with(@join-to, 'Element085')]"
		mode="stopToTransition">
		<xsl:variable name="conditionToFind">
			<xsl:apply-templates mode="rebuildCondition"
				select="parse-xml(@conditions)"></xsl:apply-templates>
		</xsl:variable>
		<Transition>
			<xsl:attribute name="stopElement" select="@join-to"></xsl:attribute>
			<xsl:attribute name="actionId"
				select="$maxActionId + number(substring-after(@join-to, 'Element085'))"></xsl:attribute>
			<xsl:call-template name="findTransition">
				<xsl:with-param name="forElement" select="."></xsl:with-param>
				<xsl:with-param name="withCondition">
					<xsl:apply-templates mode="serialize"
						select="$conditionToFind"></xsl:apply-templates>
				</xsl:with-param>
			</xsl:call-template>
		</Transition>
	</xsl:template>
	<xsl:template name="findTransition">
		<xsl:param name="forElement"></xsl:param>
		<xsl:param name="withCondition"></xsl:param>
		<xsl:variable name="transiationSvgId">
			<xsl:value-of select="$forElement/../@id"></xsl:value-of>
		</xsl:variable>
		<xsl:variable name="joinFrom"
			select="$forElement/@join-from"></xsl:variable>
		<xsl:choose>
			<xsl:when
				test="exists(/Workflow/Transitions/Transition[@SVGid=$transiationSvgId])">
				<!-- find To element -->
				<xsl:for-each
					select="/Workflow/Transitions/Transition[@SVGid=$transiationSvgId]/To[@action='999']">
					<xsl:variable name="conditionToCheck">
						<xsl:apply-templates mode="serialize" select="Condition"></xsl:apply-templates>
					</xsl:variable>
					<xsl:if test="$conditionToCheck = $withCondition">
						<xsl:attribute name="priority" select="@priority"></xsl:attribute>
						<xsl:attribute name="SVGid" select="$transiationSvgId"></xsl:attribute>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:when
				test="exists(/Workflow/Graphics/svg/g/polyline[@join-to = $joinFrom])">
				<xsl:call-template name="findTransition">
					<xsl:with-param name="forElement"
						select="/Workflow/Graphics/svg/g/polyline[@join-to = $joinFrom]"></xsl:with-param>
					<xsl:with-param name="withCondition"
						select="$withCondition"></xsl:with-param>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="*" mode="serialize">
		<xsl:text>&lt;</xsl:text>
		<xsl:value-of select="local-name()"></xsl:value-of>
		<xsl:text>&gt;</xsl:text>
		<xsl:for-each select="@*">
			<xsl:sort select="local-name()"></xsl:sort>
			<xsl:apply-templates mode="#current" select="."></xsl:apply-templates>
		</xsl:for-each>
		<xsl:apply-templates mode="#current"
			select="node()"></xsl:apply-templates>
		<xsl:text>&lt;/</xsl:text>
		<xsl:value-of select="local-name()"></xsl:value-of>
		<xsl:text>&gt;</xsl:text>
	</xsl:template>
	<xsl:template match="@*" mode="serialize">
		<xsl:text>&#032;</xsl:text>
		<xsl:value-of select="local-name()"></xsl:value-of>
		<xsl:text>="</xsl:text>
		<xsl:value-of select="."></xsl:value-of>
		<xsl:text>"</xsl:text>
	</xsl:template>
	<xsl:template match="text()" mode="serialize">
		<xsl:if test="string-length(normalize-space(.)) &gt; 0">
			<xsl:copy-of select="."></xsl:copy-of>
		</xsl:if>
	</xsl:template>
	<xsl:template match="processing-instruction()|comment()"
		mode="serialize"></xsl:template>
	<xsl:template match="/Conditions/complex"
		mode="rebuildCondition">
		<Condition>
			<xsl:apply-templates mode="#current"
				select="node()"></xsl:apply-templates>
		</Condition>
	</xsl:template>
	<xsl:template match="and|or|not" mode="rebuildCondition">
		<xsl:copy>
			<xsl:apply-templates mode="#current"
				select="node()"></xsl:apply-templates>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="/Conditions/complex//condition"
		mode="rebuildCondition">
		<xsl:variable name="id" select="@id"></xsl:variable>
		<xsl:variable name="condition"
			select="/Conditions/condition[@id=$id]"></xsl:variable>
		<xsl:variable name="elementName">
			<xsl:choose>
				<xsl:when
					test="lower-case($condition/@type) = 'placeholders'">
					<xsl:text>Placeholder</xsl:text>
				</xsl:when>
				<xsl:when
					test="lower-case($condition/@type) = 'action' or lower-case($condition/@type) = 'status'">
					<xsl:text>Action</xsl:text>
				</xsl:when>
				<xsl:when
					test="lower-case($condition/@type) = 'variable' or lower-case($condition/@type) = 'variables'">
					<xsl:text>Variable</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>Undefined</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="{$elementName}">
			<xsl:if test="$elementName = 'Placeholder'">
				<xsl:attribute name="as" select="upper-case($condition/@as)"></xsl:attribute>
			</xsl:if>
			<xsl:variable name="elementId" select="$condition/@elementID"></xsl:variable>
			<xsl:choose>
				<xsl:when test="$elementName = 'Action'">
					<xsl:attribute name="id" select="$process/Workflow/Actions/Action[@SVGid = $elementId]/@id"></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="id" select="$elementId"></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:attribute name="operation"
				select="$condition/@operator"></xsl:attribute>
			<xsl:value-of select="$condition/@result"></xsl:value-of>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
<!--Created by X4 Designer, Copyright © SoftProject GmbH. All rights reserved. -->