<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:dmn="http://www.omg.org/spec/DMN/20151101/dmn.xsd" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:default="default" version="2.0">
	<xsl:output encoding="UTF-8" method="xml"></xsl:output>
	
	<xsl:key name="caseStates" match="Case/States/State" use="Name" />
	<xsl:key name="caseFeatures" match="Case//Feature" use="Name" />

	<xsl:template match="*">
		<xsl:element name="{local-name()}" namespace="{namespace-uri()}">
			<xsl:apply-templates select="@*"></xsl:apply-templates>
			<xsl:apply-templates select="node()"></xsl:apply-templates>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="Action[Module='CaseState' or Module='TimerBasedEvent']/Parameters/Parameter[name='CaseState']/value">
		<xsl:variable name="caseState" select="." />
		<xsl:copy>
			<xsl:value-of select="key('caseStates', $caseState)/@id"/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="dmn:input[@default:type='Case Feature']/@default:expression | dmn:inputExpression[@default:type='Case Feature']/@default:expression | dmn:output[@default:type='Case Feature']/@default:expression | dmn:outputExpression[@default:type='Case Feature']/@default:expression">
		<xsl:variable name="caseFeature" select="." />
		<xsl:attribute name="{local-name()}" namespace="{namespace-uri()}" select="key('caseFeatures', $caseFeature)/@id"/>
	</xsl:template>
	
	<xsl:template match="dmn:input[@default:type='Case State']/@default:expression | dmn:inputExpression[@default:type='Case State']/@default:expression">
		<xsl:variable name="caseFeature" select="." />
		<xsl:attribute name="{local-name()}" namespace="{namespace-uri()}" select="key('caseFeatures', $caseFeature)/@id"/>
	</xsl:template>
	
	<xsl:template match="dmn:output[@default:type='Case State']/@default:expression | dmn:outputExpression[@default:type='Case State']/@default:expression">
		
	</xsl:template>
	
	<xsl:template match="dmn:inputEntry">
		<xsl:variable name="position">
			<xsl:choose>
				<xsl:when test="../dmn:description">
					<xsl:value-of select="position() - 1" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="position()" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="{local-name()}" namespace="{namespace-uri()}">
			<xsl:choose>
				<xsl:when test="../../dmn:input[@default:type='Case State' and $position]">
					<xsl:variable name="caseState" select="replace(dmn:text, '&quot;', '')" />
					<xsl:element name="text" namespace="{namespace-uri()}">"<xsl:value-of select="key('caseStates', $caseState)/@id" />"</xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:copy-of select="dmn:text" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="dmn:outputEntry">
		<xsl:variable name="position">
			<xsl:choose>
				<xsl:when test="../dmn:description">
					<xsl:value-of select="position() - 1" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="position()" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="{local-name()}" namespace="{namespace-uri()}">
			<xsl:choose>
				<xsl:when test="../../dmn:output[@default:type='Case State' and $position]">
					<xsl:variable name="caseState" select="replace(dmn:text, '&quot;', '')" />
					<xsl:element name="text" namespace="{namespace-uri()}">"<xsl:value-of select="key('caseStates', $caseState)/@id" />"</xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:copy-of select="dmn:text" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="Condition/Placeholder[@id='CASE_STATE']">
		<xsl:variable name="caseState" select="." />
		<xsl:copy>
			<xsl:apply-templates select="@*"></xsl:apply-templates>
			<xsl:value-of select="key('caseStates', $caseState)/@id" />
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="Condition/Placeholder[@id='CASE_STATE']/@id">
		<xsl:attribute name="id">
			<xsl:text>CASE_STATE_ID</xsl:text>
		</xsl:attribute>
	</xsl:template>
	
	<xsl:template match="@*">
		<xsl:copy-of select="."></xsl:copy-of>
	</xsl:template>
	
	<xsl:template match="text()|processing-instruction()|comment()">
		<xsl:copy-of select="."></xsl:copy-of>
	</xsl:template>
</xsl:stylesheet>